#include <osbind.h>
#include <gemfast.h>
#include <stddef.h>
#include <stdio.h>
#include "pulsar.h"

#define NOTE_ON 1
#define NOTE_OFF 0
#define MIDI    3
#define MIDI_READY  Bconstat(MIDI)
#define GET_MIDI    Bconin(MIDI)&0x00FF
#define SEND_MIDI(z)  Bconout(MIDI,z)
#define CLEAR_MIDI()  while(MIDI_READY)GET_MIDI /* empties buffer */

#define FORWARD 0
#define REVERSE 1
#define SHUTTLE 2
#define RANDOM  3

extern int direction[3];
extern int note[3][16];
extern int vel[3][16];
extern int chan[3];
extern int	prog[3];				/* program no for each row */
extern int	prgsel[3];			/* prog change on/off			*/
extern int first[3], last[3];
extern int noteplaying[3];
extern int trans[3];
extern int mode[3];


#define RSRCNAME  "pulsar.rsc"

extern OBJECT  *menu1;
extern OBJECT  *forminfo;
extern OBJECT  *deskform;

play(number,row)
int number,row;
{
int next,box;
unsigned char midiword[3];


switch(row)
  {
  case  0:
    box = R1BOX1;break;
  case 1:
    box = R2BOX1;break;
  case 2:
    box = R3BOX1;break;
  default:
    break;
  }

switch(direction[row])
  {
  case FORWARD:
    {
    next = number + 1;
    if(next > (last[row] - 1)) 
    	next = first[row] - 1; /* first & last set by user */
    break;
    }

  case REVERSE:
    {
    next = number - 1;
    if(next < (first[row] - 1)) next = last[row] - 1;
    break;
    }

  case SHUTTLE:
    {
    if(mode[row] == FORWARD)
      {
      next = number +1;
      if(next > (last[row] - 1))
        {
        next = last[row] - 2;
        mode[row] = REVERSE;
        }
      }
    else
      {
      next = number -1;
      if(next < (first[row] - 1))
        {
        next = first[row];
        mode[row] = FORWARD;
        }
      }
    break;
  }

  case RANDOM:
    {
    do{
      next = rand()/2000; /* rand() returns 0 to 32670 */
      }while((next < (first[row] - 1)) || (next > (last[row] - 1)));
    break;
    }
  }
obj_stchange(deskform, box+number, ~SELECTED, TRUE);

/* send last note off */
midiword[0] = 0x80 + (chan[row]-1);
midiword[1] = noteplaying[row];
midiword[2] = 0;
Midiws(2,midiword);

noteplaying[row] = note[row][next] + trans[row];
while(noteplaying[row] > 127)noteplaying[row] -= 12;
while(noteplaying[row] < 0)  noteplaying[row] += 12;

obj_stchange(deskform, box+next, SELECTED, TRUE);
/* send Midi next note on */
midiword[0] = 0x90 + (chan[row]-1);
midiword[1] = noteplaying[row];
midiword[2] = vel[row][next];
Midiws(2,midiword);

return(next);
}


turn_off(note_x,row)
int note_x,row;
{
unsigned char midiword[3];

switch(row)
  {
  case  0:
    obj_stchange(deskform, R1BOX1+note_x, ~SELECTED, TRUE);break;
  case  1:
    obj_stchange(deskform, R2BOX1+note_x, ~SELECTED, TRUE);break;
  case  2:
    obj_stchange(deskform, R3BOX1+note_x, ~SELECTED, TRUE);break;
  default:
    break;
  }

/* send midi note off */
midiword[0] = 0x80 + chan[row]-1;
midiword[1] = noteplaying[row];
midiword[2] = 0;
Midiws(2,midiword);

}


midi_enter(row)
int	row;
{
int n, temp, status = 0, inbyte, box;
unsigned char	midiword[3];

CLEAR_MIDI();		/* empty midi buffer */

if(row == 0)row = 3;	/* because last entry in menu is zero ...*/
if((row >= 1)&&(row <=3))
	frm_qtext("\x7fHit any key to cancel\n\n" "\x7fPlay notes for row %d",row);

row -=1;	/* because menu numbers entries from 1 */
switch(row)
{
	case 0:
		box=R1NOTE1;
		break;
	case 1:
		box=R2NOTE1;
		break;
	case 2:
		box=R3NOTE1;
		break;
	default: return(0);break;
}
 
for(n=0;n<16;n++)
{
	while(!MIDI_READY){if(kbhit()) return(0);}	
	

/*	...read the lower byte of input port, and reset the duart...	*/
		temp = ((GET_MIDI)&0xff);	

/**	..only process byte further if it is not system real time byte	**/
		if(( temp & 0xf8 ) != 0xf8 )
		{

/*	..if byte value < 128, then is not a status byte therefore	*/
/*	  	must be a data byte, with running status.		*/ 
			if ( temp < 128 )
			{

/*	.. so leave midiword[0] (status) unaltered...			*/
				switch(midiword[0])
				{

/*	.. and the current byte is first data byte. Following types of	*/
/*		message need two data bytes, so get another one..	*/
					case 0x90:
					case 0x80:
					case 0xe0:
					case 0xa0:
					case 0xb0: 	midiword[1] = temp;
						midiword[2] = get_midi_byte();
						break;

/*	.. following messages need only the one ( current ) data byte	*/
					case 0xd0:
					case 0xc0:	midiword[1] = temp;
							break;
					default	:	break;
				}
			}
			

/***	Otherwise, current byte is new status byte. Switch on its value, ***/
/***	ANDing with 0xf0 so that MIDI channel info is ignored.	         ***/
			else
			{
				switch(temp & 0xf0)
				{

/**	Current byte is midiword[0] ( status ), then the following 	**/
/**	message types need two data bytes ...				**/
					case 0x90:
					case 0x80:
					case 0xe0:
					case 0xa0:
					case 0xb0: 	midiword[0] = temp;
						midiword[1] = get_midi_byte();
						midiword[2] = get_midi_byte();
						break;

/**	..and the following messages need only the one data byte..	**/
					case 0xc0 :
					case 0xd0 :	midiword[0] = temp;
						midiword[1] = get_midi_byte();
						break;
					default	:	break;
				}
			}		
					
/***	Then process the complete MIDI message according to type...	***/

/**	AND midiword[0] with 0xf0 to ignore channel identification..	**/
			switch( midiword[0] & 0xf0 )
			{
				case	0x90:  /* note on */
				{
					if(midiword[2]==0){;}
					else
					{
						note[row][n] = midiword[1];
						scale(row,n,box+n);
						vel[row][n] = midiword[2];
					}
					break;
				}
				default:
				{
					n -= 1;  /* ignore */
					break;
				}
			}
 		}
 	}
}



/****			GET_MIDI_BYTE()					****/
/***	Function polls MIDI interface until a data byte may be read	***/

get_midi_byte()
{
int	databyte;

do{
		while(!MIDI_READY) {;}

/*	read only the lower byte -- upper byte is status information	*/
		databyte = ((GET_MIDI) & 0xff );	

/*	If 'databyte' is not a databyte, ( ie if status / system     	*/
/*	real time etc )	then repeat ...					*/
		} while( databyte > 127 );


/*	return value of the data byte to calling function..		*/
	return( databyte );
}


send_pr_change(ref)
int	ref;
{
unsigned char	pr_ch;

pr_ch = 0xc0 + (chan[ref]-1);   /* program change + Midi channel */

SEND_MIDI(pr_ch);
SEND_MIDI(prog[ref]);
}
